/*
 * Program:     Printset for Unix v1.0
 * Module:      AUTO
 * Description: Configure interfaces that have a default IP address
 * Date:        5-April-95
 * Author:      Ian Chard
 *
 * Copyright (C) 1993-1995 ITC Ltd.  All rights reserved.
 *
 * THIS IS UNPUBLISHED PROPRIETRY SOURCE CODE OF ITC LTD, AND MAY
 * NOT BE DISTRIBUTED, DUPLICATED OR MODIFIED WITHOUT THE WRITTEN
 * PERMISSION OF AN AUTHORISED REPRESENTATIVE OF HBM LTD.
 *
 */

static char sccsid[]="@(#)auto.c	1.6 7/18/95 ITC";

#include <curses.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "sparkle.h"

#ifdef SPARKLE_AUTO_IP

#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <net/if.h>
#include <net/if_arp.h>

#if STDC_HEADERS
# include <string.h>
#else
# ifndef HAVE_MEMCPY
#  define memcpy(d,s,n) bcopy((s),(d),(n))
#  define memmove(d,s,n) bcopy((s),(d),(n))
# endif
#endif

#ifndef HAVE_ST_ICMP
struct icmp
{
	u_char	icmp_type;
	u_char	icmp_code;
	u_short	icmp_cksum;
	n_short	icmp_id;
	n_short	icmp_seq;
};
#endif


static void ping(addr)
	struct in_addr *addr;
{
	int s;
	struct sockaddr_in dest_addr;
	struct icmp icmp;

	/* Send an ICMP_ECHO message to an interface */

	if((s=socket(AF_INET, SOCK_RAW, IPPROTO_ICMP))<0)
		fatal("raw socket: %s", sys_errlist[errno]);

	icmp.icmp_type		= ICMP_ECHO;
	icmp.icmp_code		= 0;		/* RFC 792 */
	icmp.icmp_cksum		= 0;
	icmp.icmp_id		= 0;
	icmp.icmp_seq		= 0;

	dest_addr.sin_family	= AF_INET;
	dest_addr.sin_addr	= *addr;

	if(sendto(s, &icmp, sizeof(icmp), 0, (struct sockaddr *) &dest_addr, sizeof(dest_addr))<0)
		fatal("icmp sendto: %s", sys_errlist[errno]);

	close(s);
}


static int insert_arp_entry(node, addr)
	char *node;
	struct in_addr *addr;
{
	int s;
	struct arpreq arpreq;
	struct sockaddr_in new_addr;

	memset(&new_addr, 0, sizeof(new_addr));
	memset(&arpreq, 0, sizeof(arpreq));

	s=socket(AF_INET, SOCK_DGRAM, 0);

	new_addr.sin_addr	= *addr;

	/* Hackity-hack - don't ask me why this +2 is needed */

	arpreq.arp_pa.sa_family	= AF_INET;
	memcpy(((char *) arpreq.arp_pa.sa_data)+2, &new_addr.sin_addr, sizeof(new_addr.sin_addr));

	arpreq.arp_ha.sa_family	= AF_UNSPEC;
	memcpy(arpreq.arp_ha.sa_data, node, 6);

	arpreq.arp_flags=ATF_COM;

	if(ioctl(s, SIOCSARP, (caddr_t) &arpreq)<0)
		fatal("ioctl SIOCSARP", sys_errlist[errno]);

	close(s);

	/* We never need to delete this entry, because it's
	   not permanent and will time out eventually. */

	return 0;
}


void auto_ip(unit)
	struct interface *unit;
{
	struct in_addr new_addr;

	clear();
	timer_state(0);

	centre(1, "Initial IP address configuration");

	/* Are we superuser?  Can't use SOCK_RAW if we're not. */

	if(geteuid()!=0)
	{
		centre(LINES/2, "This program must run as superuser to use Auto IP.");

		press_any_key();
		return;
	}

	/* First check if Auto IP is actually switched on... */

	if(!unit->ps.AutoIP)
	{
		centre(LINES/2, "This unit does not have Auto IP switched on.");
		centre(LINES/2+1, "It cannot be configured by this utility.");

		press_any_key();
		return;
	}

	centre(LINES/2-5, "Please enter an IP address to give this printer.");
	echo();
	do
	{
		move(LINES/2, 0);
		clrtoeol();
		standout();
		mvprintw(LINES/2, COLS/2-8, "               ");
		refresh();
		mvgetstr(LINES/2, COLS/2-8, scratch_buffer);
		standend();
		if(*scratch_buffer=='\0')
			return;
	} while((new_addr.s_addr=inet_addr(scratch_buffer))==-1);
	noecho();

	if(insert_arp_entry(unit->ps.Node, new_addr)==0)
		ping(new_addr);

	press_any_key();
}

#else

void auto_ip(unit)
	int unit;
{ }

#endif /* SPARKLE_AUTO_IP */

