/*
 * Program:	Printset for Unix v1.0
 * Module:	DISPLAY
 * Description:	Initialise and handle curses display
 * Date:	3-March-95
 * Author:	Ian Chard
 *
 * Copyright (C) 1993-1995 ITC Ltd.  All rights reserved.
 *
 * THIS IS UNPUBLISHED PROPRIETRY SOURCE CODE OF ITC LTD, AND MAY
 * NOT BE DISTRIBUTED, DUPLICATED OR MODIFIED WITHOUT THE WRITTEN
 * PERMISSION OF AN AUTHORISED REPRESENTATIVE OF HBM LTD.
 *
 */

static char sccsid[]="@(#)display.c	1.12 7/18/95 ITC";

#include <curses.h>
#include <time.h>
#ifdef TM_IN_SYS_TIME
# include <sys/time.h>
#endif
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "sparkle.h"

WINDOW *wMain, *wList;
static struct
{
	char *status;
	int error;
} status[8];


static void restore_cursor_position()
{
	move(LINES-1, COLS/2+5);	/* doesn't always work for some reason */
	refresh();
}


char *printer_status(ps, error)
	PSDetailInfo *ps;
	int *error;
{
	int i, statbit;

	*error=0;

	if(ps->PStatus==0xff)
		return "Unavailable";

	if(ps->StatusStringReceived)
		return ps->StatusString;

	if(ps->Printing)
		return "Printing";

	if(ps->PStatus==0)
		return "Idle";

	for(i=0; i<8; i++)
		if(ps->PStatus & (1<<i))
			statbit=i;

	*error=status[statbit].error;

	return status[statbit].status;
}


void display_init()
{
	wMain=initscr();
	noecho();
	cbreak();

#ifdef HAVE_INTRFLUSH
	intrflush(stdscr, FALSE);
#endif

#ifdef HAVE_KEYPAD
	keypad(stdscr, TRUE);
#endif

	wList=newwin(LINES-7, COLS-6, 3, 2);

	status[0].status="Off line";		status[0].error=0;
	status[1].status="Paper empty";		status[1].error=0;
	status[2].status="Paper jam";		status[2].error=1;
	status[3].status="Toner empty";		status[3].error=1;
	status[4].status="Not ready";		status[4].error=0;
	status[5].status="Cover open";		status[5].error=1;
	status[6].status="Service call";	status[6].error=1;
	status[7].status="Warming up";		status[7].error=0;
}


void draw_screen()
{
	clear();
	timer_state(1);

	centre(0, "PRINTSET FOR UNIX version 1.0");
	centre(1, "Copyright (C) 1993-1995 ITC Ltd");
	centre(LINES-3, "k=up, j=down, RETURN=configure, $=refresh, q=quit");
	centre(LINES-1, "Command: ");

	refresh();

	display_list();
}


static void display_row(row, highlight)
	int row;
	int highlight;
{
	static int error;

	strcpy(scratch_buffer, printer_status(&SortedList[row]->ps, &error));

#ifdef A_BLINK
	if(error)
		wattron(wList, A_BLINK);
#endif

	wmove(wList, row+3, 2);
	if(highlight)
		wstandout(wList);

	wprintw(wList, "%-18.18s %-15.15s %02X:%02X:%02X:%02X:%02X:%02X  %-13.13s %2d",
		SortedList[row]->ps.PSName,
		inet_ntoa(SortedList[row]->addr),
		SortedList[row]->ps.Node[0],
		SortedList[row]->ps.Node[1],
		SortedList[row]->ps.Node[2],
		SortedList[row]->ps.Node[3],
		SortedList[row]->ps.Node[4],
		SortedList[row]->ps.Node[5],
		scratch_buffer,
		SortedList[row]->alive
		);

	if(highlight)
		wstandend(wList);

#ifdef A_BLINK
	if(error)
		wattroff(wList, A_BLINK);
#endif
}


void display_list()
{
	int i;

#ifdef HAVE_STRFTIME
	struct tm *tm;
	time_t now;

	time(&now);
	tm=localtime(&now);
	strftime(scratch_buffer, 16, "%H:%M:%S", tm);
	mvprintw(0, 0, scratch_buffer);
#endif /* HAVE_STRFTIME */

	werase(wList);
	box(wList,'*','*');

	mvwprintw(wList, 1, 2, "Name               IP address      Ethernet address   Status        CD");

	for(i=0; i<NumberOfSummaries; i++)
		display_row(i, 0);

	if(current_row==-1 && NumberOfSummaries)
		current_row=0;

	highlight_row(current_row);

	restore_cursor_position();
	wrefresh(wList);
}


void highlight_row(row)
	int row;
{
	if(row<NumberOfSummaries && row!=-1)
	{
		display_row(current_row, 0);
		display_row(row, 1);

		current_row=row;
	}
	else
		current_row=-1;
	
	restore_cursor_position();
}


void display_shutdown()
{
	mvcur(0, 0, LINES, 0);
	endwin();
}

