/*
 * Program:     Printset for Unix v1.0
 * Module:      INTERFACE
 * Description: Display and modify interface parameters
 * Date:        3-March-95
 * Author:      Ian Chard
 *
 * Copyright (C) 1993-1995 ITC Ltd.  All rights reserved.
 *
 * THIS IS UNPUBLISHED PROPRIETRY SOURCE CODE OF ITC LTD, AND MAY
 * NOT BE DISTRIBUTED, DUPLICATED OR MODIFIED WITHOUT THE WRITTEN
 * PERMISSION OF AN AUTHORISED REPRESENTATIVE OF HBM LTD.
 *
 */

static char sccsid[]="@(#)interface.c	1.7 7/18/95 ITC";

#include <curses.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "sparkle.h"

/* The display consists of two distinct lists (although they may appear
   continuous) - values, and parameters.  The values are not configurable
   and serve to identify the unit.  The parameters are the configurable
   options. */

static int current_param;	/* currently selected parameter */
static int param_top;		/* position of first parameter or value */
static int param_count;		/* total number of parameters (excl values) */
static int first_param;		/* first configurable parameter */
static int changes_made;	/* have changes been made? */
static int current_page;	/* current parameter page */
static int params_per_page;	/* number of parameters & values per page */


enum types
{
	TYPE_STRING,
	TYPE_NUMERIC,
	TYPE_BOOLEAN,
	TYPE_ETHERNET,
	TYPE_IPADDR,
	TYPE_SERIALNO,

	TYPE_NULL
};

static struct
{
	char		*desc;
	enum types	type;
	off_t		offset;
} param_table[] =
{
	"Unit information",	TYPE_NULL,	0,

	"Hardware type",	TYPE_STRING,	offsetof(PSDetailInfo, HardwareDescription),
	"Ethernet address",	TYPE_ETHERNET,	offsetof(PSDetailInfo, Node),
	"Serial no",		TYPE_SERIALNO,	offsetof(PSDetailInfo, Node),
	"Firmware version",	TYPE_STRING,	offsetof(PSDetailInfo, Version),

	NULL,			0,		0,					/* marks end of read-only values */


	"General parameters",	TYPE_NULL,	0,

	"General name",		TYPE_STRING,	offsetof(PSDetailInfo, PSName),
	"Busy timeout",		TYPE_NUMERIC,	offsetof(PSDetailInfo, BusyTime),
	"Sharer timeout",	TYPE_NUMERIC,	offsetof(PSDetailInfo, ShareTime),


	"Netware parameters",	TYPE_NULL,	0,

	"Novell name",		TYPE_STRING,	offsetof(PSDetailInfo, NWPSName),
	"Fileserver",		TYPE_STRING,	offsetof(PSDetailInfo, FSName),
	"Hunting",		TYPE_BOOLEAN,	offsetof(PSDetailInfo, Hunting),
	"Remote printer #",	TYPE_NUMERIC,	offsetof(PSDetailInfo, Printer),


	"Appletalk parameters",	TYPE_NULL,	0,

	"Appletalk name",	TYPE_STRING,	offsetof(PSDetailInfo, ATPSName),
	"Active zone",		TYPE_STRING,	offsetof(PSDetailInfo, ATZone),


	"TCP/IP parameters",	TYPE_NULL,	0,

	"IP address",		TYPE_IPADDR,	offsetof(PSDetailInfo, IPAddr),
	"Subnet mask",		TYPE_IPADDR,	offsetof(PSDetailInfo, SubnetMask),
	"Auto IP",		TYPE_BOOLEAN,	offsetof(PSDetailInfo, AutoIP),


	NULL,			0,		0					/* marks end of table */
};


static char *format_field(ps, param)
	PSDetailInfo *ps;
	int param;
{
	char *ptr;

	ptr=(char *) ((char *) ps)+param_table[param].offset;

	switch(param_table[param].type)
	{
		case TYPE_STRING:
			strcpy(scratch_buffer, ptr);
			break;

		case TYPE_NUMERIC:
			sprintf(scratch_buffer, "%d", *ptr);
			break;

		case TYPE_ETHERNET:
			sprintf(scratch_buffer, "%02x:%02x:%02x:%02x:%02x:%02x",
				(unsigned char) ptr[0],
				(unsigned char) ptr[1],
				(unsigned char) ptr[2],
				(unsigned char) ptr[3],
				(unsigned char) ptr[4],
				(unsigned char) ptr[5]);
			break;

		case TYPE_SERIALNO:
			sprintf(scratch_buffer, "%02X%02X%01X%c-%01X%01X",
				(unsigned char) ptr[3],
				(unsigned char) ptr[4],
				(unsigned char) ptr[5] / 16,
				ps->HWType & TOKENRING? 'R':'E',
				0x09,
				(unsigned char) ptr[5] & 0x0f);
			break;

		case TYPE_IPADDR:
			sprintf(scratch_buffer, "%u.%u.%u.%u", (unsigned char) ptr[0], (unsigned char) ptr[1], (unsigned char) ptr[2], (unsigned char) ptr[3]);
			break;

		case TYPE_BOOLEAN:
			strcpy(scratch_buffer, *(unsigned char *) ptr? "Yes":"No");
			break;

		default:
			strcpy(scratch_buffer, "Modem!");
	}

	return scratch_buffer;
}


static int store_field(ps, param, data)
	PSDetailInfo *ps;
	int param;
	void *data;
{
	char *ptr;
	unsigned long inaddr;

	ptr=(char *) ((char *) ps)+param_table[param].offset;

	switch(param_table[param].type)
	{
		case TYPE_STRING:
			strcpy(ptr, (char *) data);
			return 1;
		
		case TYPE_IPADDR:
			if((inaddr=inet_addr(data))==-1)
				return 0;

			bcopy(&inaddr, ptr, 4);
			return 1;
		
		case TYPE_BOOLEAN:
			*ptr=*(unsigned char *) data;
			return 1;

		default:
			break;
	}
	
	return 0;
}


static void display_param(ps, param, row)
	PSDetailInfo *ps;
	int param;
	int row;
{
	if(param_table[param].type==TYPE_NULL)
	{
		standout();
		mvprintw(row, 0, param_table[param].desc);
		standend();
	}
	else
		mvprintw(row, 3, "%-27.27s %-40.40s", param_table[param].desc, format_field(ps, param));
}


static void input_param(ps, param)
	PSDetailInfo *ps;
	int param;
{
	move(param_top+param, 30);
	clrtoeol();
	standout();
	echo();

	switch(param_table[param].type)
	{
		case TYPE_BOOLEAN:
			*scratch_buffer=*(unsigned char *) (((char *) ps)+param_table[param].offset)? 0:1;
			break;

		default:
			move(param_top+param, 31);
			refresh();
			getstr(scratch_buffer);
			strupr(scratch_buffer);
			break;
	}

	noecho();

	if(store_field(ps, param, scratch_buffer))
	{
		changes_made=1;
		mvaddch(param_top+param, 2, '*');
	}

	display_param(ps, param, param_top+param);
	standend();
}


static void highlight_param(ps, param)
	PSDetailInfo *ps;
	int param;
{
	if(param<0)
		return;

	display_param(ps, current_param+first_param, current_param+first_param+param_top);
	standout();
	display_param(ps, param+first_param, param+first_param+param_top);
	standend();

	current_param=param;
}


static int save_params(ps)
	PSDetailInfo *ps;
{
	int result;

	clear();

	if(changes_made)
	{
		if((result=SendDetailPacket(ps)))
		{
			centre(LINES/2-1, "Details have been saved to the interface.");
			centre(LINES/2+1, "There may be a few seconds' delay while the interface");
			centre(LINES/2+2, "resets itself before the changes appear.");
		}
		else
			centre(LINES/2, "No response from printserver.");
	}
	else
		centre(LINES/2, "No changes were made to the printserver.");

	press_any_key();

	return result;
}


static void draw_details_screen(ps)
	PSDetailInfo *ps;
{
	int i,row;

	clear();

	centre(0, "Configuration details for \"%s\" (page %d)", ps->PSName, current_page+1);
	centre(LINES-1, "k=up, j=down, RETURN=edit parameter, s=save, q=quit without save");

	/* Draw whatever page we're in */

	for(i=current_page*params_per_page, row=2; param_table[i].desc; i++, row++)
		display_param(ps, i, row);

	first_param=++i;
	param_top=row-first_param;
	for(param_count=0; param_table[i].desc; i++, param_count++)
		display_param(ps, i, param_top+i);

	highlight_param(ps, 1);

	refresh();
}


void display_details(unit)
	struct interface *unit;
{
	int i,done=0;
	PSDetailInfo ps;

	clear();
	timer_state(0);

	params_per_page=LINES-5;

	memcpy(&ps, &unit->ps, sizeof(ps));
	draw_details_screen(&ps);

	changes_made=0;

	while(!done)
	{
		while((i=getch())==-1);

		switch(i)
		{
			case 'j':
#ifdef KEY_DOWN
			case KEY_DOWN:
#endif

				if(current_param==param_count-1)
				{
#ifdef HAVE_BEEP
					beep();
#endif
				}
				else if(current_param==(current_page*params_per_page)-1)
				{
					current_page++;
					draw_details_screen(&ps);
					highlight_param(&ps, current_param+1+(param_table[current_param+first_param+1].type==TYPE_NULL? 1:0));
					refresh();
				}
				else
				{
					highlight_param(&ps, current_param+1+(param_table[current_param+first_param+1].type==TYPE_NULL? 1:0));
					refresh();
				}
				break;

			case 'k':
#ifdef KEY_UP
			case KEY_UP:
#endif

				if(current_param>0)
				{
					highlight_param(&ps, current_param-1-(param_table[current_param+first_param-1].type==TYPE_NULL? 1:0));
					refresh();
				}
				break;

			case 's':
			case 'S':

				if(!save_params(&ps))
				{
					draw_details_screen(&ps);
					break;		/* don't quit, might want to try again */
				}

				AcquirePSList();

				/*FALLTHROUGH*/

			case 'q':
			case 'Q':
				done=1;
				break;

			case '\n':
#ifdef KEY_ENTER
			case KEY_ENTER:
#endif

				input_param(&ps, current_param+first_param);
				refresh();
				break;

#ifdef HAVE_BEEP
			default:
				beep();
#endif
		}
	}
}
