/*
 * Program:     Printset for Unix v1.0
 * Module:      LIST
 * Description: Maintain list of interfaces
 * Date:        4-April-95
 * Author:      Ian Chard
 *
 * Copyright (C) 1993-1995 ITC Ltd.  All rights reserved.
 *
 * THIS IS UNPUBLISHED PROPRIETRY SOURCE CODE OF ITC LTD, AND MAY
 * NOT BE DISTRIBUTED, DUPLICATED OR MODIFIED WITHOUT THE WRITTEN
 * PERMISSION OF AN AUTHORISED REPRESENTATIVE OF HBM LTD.
 *
 */

static char sccsid[]="@(#)list.c	1.7 7/18/95 ITC";

#include "sparkle.h"
#if STDC_HEADERS
# include <string.h>
#else
# ifndef HAVE_MEMCPY
#  define memcpy(d,s,n) bcopy((s),(d),(n))
#  define memmove(d,s,n) bcopy((s),(d),(n))
# endif
#endif


struct interface *SummaryList;		/* global interface list */
struct interface *SortedList[128];	/* pointers into SummaryList */
int		  NumberOfSummaries;	/* guess! */


/* Add an interface to the list - if it's already in there, then update its
   information and alive time. */

struct interface *add_interface_to_list(addr, ps)
	struct in_addr *addr;
	PSDetailInfo *ps;
{
	struct interface *new;

	if((new=find_interface_in_list(ps)))
	{
		memcpy(&new->addr, addr, sizeof(new->addr));
		memcpy(&new->ps, ps, sizeof(new->ps));
		memcpy(&new->ps.Address, addr, sizeof(new->ps.Address));
		new->alive=4;
		return new;
	}

	new=(struct interface *) malloc(sizeof(struct interface));

	new->prev=NULL;
	if((new->next=SummaryList))
		new->next->prev=new;

	memcpy(&new->addr, addr, sizeof(new->addr));
	memcpy(&new->ps, ps, sizeof(new->ps));
	memcpy(&new->ps.Address, addr, sizeof(new->ps.Address));

	new->alive=4;

	SummaryList=new;

	return SummaryList;
}


struct interface *find_interface_in_list(ps)
	PSDetailInfo *ps;
{
	struct interface *current;

	for(current=SummaryList; current; current=current->next)
		if(memcmp(current->ps.Node, ps->Node, 6)==0)
			return current;
	
	return NULL;
}


static int compare_interfaces(p1, p2)
	const void *p1;
	const void *p2;
{
	return strcmp((*(struct interface **) p1)->ps.PSName, (*(struct interface **) p2)->ps.PSName);
}


/* Decrement all the alive counters in the list, and delete any interfaces
   that have timed out, and build a sorted list of pointers ready to be
   displayed. */

void update_list()
{
	struct interface *current, *old;

	for(current=SummaryList, NumberOfSummaries=0; current; )
		if(!--current->alive)
		{
			/* Timed out - delete it */

			if(current->prev)
				current->prev->next=current->next;
			if(current->next)
				current->next->prev=current->prev;
			if(SummaryList==current)
				SummaryList=SummaryList->next;
			
			old=current;
			current=current->next;
			free(old);
		}
		else
		{
			/* Add it to the array of pointers */

			SortedList[NumberOfSummaries++]=current;
			current=current->next;
		}
	
	qsort(SortedList, NumberOfSummaries, sizeof(struct interface *), compare_interfaces);
}

