/*
 * Program:     Printset for Unix v1.0
 * Module:      UDP
 * Description: Retrieve and set interface parameters
 * Date:        3-March-95
 * Author:      Ian Chard
 *
 * Copyright (C) 1993-1995 ITC Ltd.  All rights reserved.
 *
 * THIS IS UNPUBLISHED PROPRIETRY SOURCE CODE OF ITC LTD, AND MAY
 * NOT BE DISTRIBUTED, DUPLICATED OR MODIFIED WITHOUT THE WRITTEN
 * PERMISSION OF AN AUTHORISED REPRESENTATIVE OF HBM LTD.
 *
 */

static char sccsid[]="@(#)udp.c	1.7 7/18/95 ITC";

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include <pwd.h>
#include <fcntl.h>
#ifdef sun
#include <sys/filio.h>
#endif
#include "sparkle.h"
#include "pssumry.h"

static int s, optval=1, NumberOfBroadcastNetworks, port;
static const char *probe="HBM\001probe", *psetrc=".psetrc";
static char data[12];
static unsigned long BroadcastNetworks[256];	/* that's a lorra lorra networks */


int UDPInit()
{
	struct sockaddr_in local_addr;
	struct passwd *pw;
	FILE *fp;
	int i,bound=0;

	if((s=socket(AF_INET, SOCK_DGRAM, 0))<0)
	{
		fatal("socket: %s", sys_errlist[errno]);
		return 0;
	}

	setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval));
	if(fcntl(s, F_SETFL, FNDELAY)<0)
		fatal("fcntl FNDELAY: %s", sys_errlist[errno]);

	local_addr.sin_family=AF_INET;
	local_addr.sin_addr.s_addr=htonl(INADDR_ANY);

	for(i=8450; i<8450+30; i++)
	{
		local_addr.sin_port=htons(i);
		if((bind(s, (struct sockaddr *) &local_addr, sizeof(local_addr)))==0)
		{
			bound=1;
			break;
		}
		else if(errno!=EADDRINUSE)
			fatal("unexpected error from bind: %s", sys_errlist[errno]);
	}

	if(!bound)
		fatal("Couldn't find a free UDP port!\nAre more than %d copies of this program running?\n", 30);

	port=i;
	memcpy(data+2, probe, strlen(probe));
	data[0]=port/256;
	data[1]=port%256;

	/* Read the user's configuration file, if any */

	if(!(pw=getpwuid(getuid())))
		fatal("Can't seem to get user information for uid %d", getuid());

	sprintf(scratch_buffer, "%s/%s", pw->pw_dir, psetrc);
	if(!(fp=fopen(scratch_buffer, "r")))
#ifdef SYSTEM_PSETRC
		if(!(fp=fopen(SYSTEM_PSETRC, "r")))
#endif
			fatal("Can't find a configuration file.");

	while(!feof(fp))
	{
		fgets(scratch_buffer, 80, fp);
		if((BroadcastNetworks[NumberOfBroadcastNetworks]=inet_addr(scratch_buffer)))
			NumberOfBroadcastNetworks++;
	}
	
	fclose(fp);

	if(NumberOfBroadcastNetworks==0)
		fatal("Configuration file contains no valid networks.");

	/* Add the default network (local segment only) */

	BroadcastNetworks[NumberOfBroadcastNetworks++]=htonl(0);

	return 1;
}


void AcquirePSList()
{
	struct sockaddr_in dest_addr, src_addr;
	int optval=1, len, sizeof_sockaddr_in, i;
	static PSDetailInfo ps;

	setsockopt(s, SOL_SOCKET, SO_BROADCAST, &optval, sizeof(optval));

	dest_addr.sin_family=AF_INET;
	dest_addr.sin_port=htons(8450);

	for(i=0; i<NumberOfBroadcastNetworks; i++)
	{
		dest_addr.sin_addr.s_addr=BroadcastNetworks[i];
		if(sendto(s, data, strlen(data), 0, (struct sockaddr *) &dest_addr, sizeof(dest_addr))<0)
			fatal("sendto broadcast: %s", sys_errlist[errno]);
	}

	sleep(1);

	sizeof_sockaddr_in=sizeof(struct sockaddr_in);
	while((len=recvfrom(s, scratch_buffer, 8192, 0, (struct sockaddr *) &src_addr, &sizeof_sockaddr_in))>0)
		if(strncmp(scratch_buffer, "hbm", 3)==0)
		{
			GetDetailContents(&ps, scratch_buffer+3, len-3);
			add_interface_to_list(&src_addr.sin_addr, &ps);
		}

	if(errno!=EWOULDBLOCK)
		fatal("unexpected error from recvfrom: %s", sys_errlist[errno]);
	
	optval=0;
	setsockopt(s, SOL_SOCKET, SO_BROADCAST, &optval, sizeof(optval));

	update_list();
}


int SendDetailPacket(ps)
	PSDetailInfo *ps;
{
	int len,err;
	struct sockaddr_in dest_addr;

	len=SetDetailContents(ps, scratch_buffer+2);

	dest_addr.sin_family=AF_INET;
	dest_addr.sin_port=htons(8450);
	dest_addr.sin_addr=ps->Address;

	scratch_buffer[0]='\041';
	scratch_buffer[1]='\003';

	err=sendto(s, scratch_buffer, len+2, 0, (struct sockaddr *) &dest_addr, sizeof(dest_addr));

	return err;
}

