/*
 * Program:     Printset for Unix v1.0
 * Module:      UTILS
 * Description: Miscellany
 * Date:        3-March-95
 * Author:      Ian Chard
 *
 * Copyright (C) 1993-1995 ITC Ltd.  All rights reserved.
 *
 * THIS IS UNPUBLISHED PROPRIETRY SOURCE CODE OF ITC LTD, AND MAY
 * NOT BE DISTRIBUTED, DUPLICATED OR MODIFIED WITHOUT THE WRITTEN
 * PERMISSION OF AN AUTHORISED REPRESENTATIVE OF HBM LTD.
 *
 */

static char sccsid[]="@(#)utils.c	1.9 7/18/95 ITC";

#include <curses.h>

#ifdef HAVE_VARARGS_H
# include <varargs.h>
#else
# include <stdarg.h>
#endif

#include <signal.h>
#include "sparkle.h"


#ifndef HAVE_HALFDELAY
RETSIGTYPE alarm_handler()
{
	AcquirePSList();
	display_list();

	signal(SIGALRM, alarm_handler);
	alarm(SPARKLE_REFRESH);
}
#endif


void timer_state(state)
	int state;
{
#ifdef HAVE_HALFDELAY
	if(state)
		halfdelay(SPARKLE_REFRESH*10);	/* 1/10s */
	else
		halfdelay(-1);
#else
	if(state)
		alarm(SPARKLE_REFRESH);
	else
		alarm(0);
#endif
}


void centre(va_alist)
	va_dcl
{
	va_list va;
	int y;
	char *fmt;

	va_start(va);

	y=va_arg(va, int);
	fmt=va_arg(va, char *);

	vsprintf(scratch_buffer,fmt,va);
	va_end(va);

	mvwaddstr(wMain, y, COLS/2-strlen(scratch_buffer)/2, scratch_buffer);
}


void press_any_key()
{
	centre(LINES-2, "Press any key to continue");
	refresh();
	getch();
}


char *strupr(str)
	char *str;
{
	char *c;

	for(c=str; *c; c++)
		if(isspace(*c))
			*c='_';
		else
			*c=toupper(*c);
	
	return str;
}


RETSIGTYPE fatal_signal(sig)
	int sig;
{
#ifndef SYS_SIGLIST_DECLARED
	extern char *sys_siglist[];
#endif

	display_shutdown();
	fprintf(stderr, "\n\nFatal error:  caught signal %d (%s)!\n\n", sig, sys_siglist[sig]);
	signal(sig, SIG_DFL);
	kill(getpid(), sig);
}


RETSIGTYPE nonfatal_signal(sig)
	int sig;
{
	display_shutdown();
	signal(sig, SIG_DFL);
	kill(getpid(), sig);
}


void catch_signals()
{
	int i;

	for(i=SIGILL; i<=SIGPIPE; i++)
		signal(i, fatal_signal);
	
	signal(SIGTERM, nonfatal_signal);

#ifndef HAVE_HALFDELAY
	signal(SIGALRM, alarm_handler);
#endif
}


void fatal(va_alist)
	va_dcl
{
	va_list va;
	char *fmt;

	va_start(va);

	fmt=va_arg(va, char *);

	vsprintf(scratch_buffer, fmt, va);
	va_end(va);

	if(wMain)
		display_shutdown();

	fprintf(stderr, "\nFatal error:  %s\n\n", scratch_buffer);

	_exit(1);
}

